/// <reference types="node" />
import { BigNumber, BigNumberish } from 'ethers';
import { TokenId, ChainName, ChainId, ParsedRelayerPayload } from '../types';
import { TokenBridgeAbstract } from './tokenBridge';
/**
 * @abstract
 *
 * A standard set of methods for interacting with the Token Bridge Relayer contracts across any of the supported chains
 *
 * @example
 * const context = Wormhole.getContext(chain); // get the chain Context
 * const hasRelayer = context.relaySupported(chain);
 * if (hasRelayer) {
 *   // call any of the supported methods in a standardized uniform fashion
 *   context.startTransferWithRelay(...);
 * }
 */
export declare abstract class RelayerAbstract<TransactionResult> extends TokenBridgeAbstract<TransactionResult> {
    /**
     * Returns if the Token Bridge relayer is supported on a given chain
     *
     * @param chain The chain name or id
     * @returns True/False if the Token Bridge relayer is supported or not
     */
    relaySupported(chain: ChainName | ChainId): boolean;
    /**
     * Send a Token Bridge Relay transfer. This will automatically dispatch funds to the recipient on the destination chain.
     *
     * @param token The Token Identifier (native chain/address) or `'native'` if sending the native token
     * @param amount The token amount to be sent, as a string
     * @param toNativeToken The amount of sending token to be converted to native gas token on the destination chain
     * @param sendingChain The source chain name or id
     * @param senderAddress The address that is dispatching the transfer
     * @param recipientChain The destination chain name or id
     * @param recipientAddress The wallet address where funds will be sent (On solana, this is the associated token account)
     * @param overrides Optional overrides, varies by chain
     * @returns The transaction receipt
     */
    abstract startTransferWithRelay(token: TokenId | 'native', amount: bigint, toNativeToken: string, sendingChain: ChainName | ChainId, senderAddress: string, recipientChain: ChainName | ChainId, recipientAddress: string, overrides?: any): Promise<TransactionResult>;
    /**
     * Computes the amount of native gas tokens that would be received, given current conversion rates
     *
     * @param destChain The destination chain name or id
     * @param tokenId The token identifier (native chain/address)
     * @param amount The amount of the sending token which would be converted to native tokens on the destination chain
     * @param walletAddress The receiving wallet address
     * @returns The amount of destination gas drop-off
     */
    abstract calculateNativeTokenAmt(destChain: ChainName | ChainId, tokenId: TokenId, amount: BigNumberish, walletAddress: string): Promise<BigNumber>;
    /**
     * Gets the maximum amount of sending token which can be converted to destination native gas token
     *
     * @param destChain The destination chain name or id
     * @param tokenId The token identifier (native chain/address)
     * @param walletAddress The receiving wallet address
     * @returns The maximum swap amount
     */
    abstract calculateMaxSwapAmount(destChain: ChainName | ChainId, tokenId: TokenId, walletAddress: string): Promise<BigNumber>;
    /**
     * Gets the relayer fee (paid using sending token)
     *
     * @param sourceChain The source chain name or id
     * @param destChain The destination chain name or id
     * @param tokenId The token identifier (native chain/address)
     * @returns The relayer fee
     */
    abstract getRelayerFee(sourceChain: ChainName | ChainId, destChain: ChainName | ChainId, tokenId: TokenId): Promise<BigNumber>;
    parseRelayerPayload(transferPayload: Buffer): ParsedRelayerPayload;
}
