"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Wormhole = void 0;
const multi_provider_1 = require("@nomad-xyz/multi-provider");
const axios_1 = __importDefault(require("axios"));
const MAINNET_1 = __importStar(require("./config/MAINNET"));
const TESTNET_1 = __importStar(require("./config/TESTNET"));
const types_1 = require("./types");
const vaa_1 = require("./utils/vaa");
/**
 * The WormholeContext manages connections to Wormhole Core, Bridge and NFT Bridge contracts.
 * It inherits from the {@link MultiProvider}, and ensures that its contracts
 * always use the latest registered providers and signers.
 *
 * For convenience, we've pre-constructed contexts for mainnet, testnet and devnet
 * deployments. These can be imported directly from the wormhole sdk.
 *
 * @example
 * // Set up mainnet and then access contracts as below:
 * const context = new WormholeContext(Network.Mainnet);
 * let bridge = context.mustGetBridge('ethereum');
 *
 * // interact easily with any chain!
 * // supports EVM, Solana, Terra, etc
 * const tokenId = {
 *   chain: 'ethereum',
 *   address: '0x123...',
 * }
 * const receipt = context.startTransfer(
 *   tokenId,
 *   new BigInt(10), // amount
 *   'ethereum', // sending chain
 *   '0x789...', // sender address
 *   'moonbeam', // destination chain
 *   '0x789..., // recipient address on destination chain
 * )
 */
class Wormhole extends multi_provider_1.MultiProvider {
    constructor(network, contexts, conf) {
        super();
        if (conf) {
            this.conf = conf;
        }
        else {
            this.conf = network === types_1.Network.MAINNET ? MAINNET_1.default : TESTNET_1.default;
        }
        this._contexts = new Map();
        for (const contextType in contexts) {
            const context = new contexts[contextType](this);
            this._contexts.set(contextType, context);
        }
        this.registerProviders();
    }
    get network() {
        return this.conf.network;
    }
    /**
     * Registers evm providers
     */
    registerProviders() {
        for (const network of Object.keys(this.conf.rpcs)) {
            const n = network;
            const chains = this.conf.network === types_1.Network.MAINNET ? MAINNET_1.MAINNET_CHAINS : TESTNET_1.TESTNET_CHAINS;
            const chainConfig = chains[n];
            if (!chainConfig)
                throw new Error('invalid network name');
            // register domain
            this.registerDomain({
                // @ts-ignore
                domain: chainConfig,
                name: network,
            });
            // register RPC provider
            if (this.conf.rpcs[n]) {
                this.registerRpcProvider(network, this.conf.rpcs[n]);
            }
        }
    }
    /**
     * Converts to chain id
     * @param nameOrId the chain name or chain id
     * @returns the chain id
     */
    toChainId(nameOrId) {
        return super.resolveDomain(nameOrId);
    }
    /**
     * Converts to chain name
     * @param nameOrId the chain name or chain id
     * @returns the chain name
     */
    toChainName(nameOrId) {
        return super.resolveDomainName(nameOrId);
    }
    /**
     * Gets the contract addresses for a given chain
     * @param chain the chain name or chain id
     * @returns the contract addresses
     */
    getContracts(chain) {
        const chainName = this.toChainName(chain);
        return this.conf.chains[chainName]?.contracts;
    }
    /**
     * Gets the contract addresses for a given chain
     * @param chain the chain name or chain id
     * @returns the contract addresses
     * @throws Errors if contracts are not found
     */
    mustGetContracts(chain) {
        const contracts = this.getContracts(chain);
        if (!contracts)
            throw new Error(`no contracts found for ${chain}`);
        return contracts;
    }
    /**
     * Returns the chain "context", i.e. the class with chain-specific logic and methods
     * @param chain the chain name or chain id
     * @returns the chain context class
     * @throws Errors if context is not found
     */
    getContext(chain) {
        const chainName = this.toChainName(chain);
        const { context: contextType } = this.conf.chains[chainName];
        const context = this._contexts.get(contextType);
        if (!context)
            throw new Error('Not able to retrieve context');
        return context;
    }
    async getSolanaRecipientAddress(recipientChain, tokenId, recipientAddress) {
        const recipientChainId = this.toChainId(recipientChain);
        if (recipientChainId === MAINNET_1.MAINNET_CHAINS.solana) {
            let solanaContext;
            try {
                solanaContext = this.getContext(MAINNET_1.MAINNET_CHAINS.solana);
            }
            catch (_) {
                throw new Error('You attempted to send a transfer to Solana, but the Solana context is not registered. You must import SolanaContext from @wormhole-foundation/connect-sdk-solana and pass it in to the Wormhole class constructor');
            }
            const account = await solanaContext.getAssociatedTokenAddress(tokenId, recipientAddress);
            return account.toString();
        }
    }
    /**
     * Fetches the address for a token representation on any chain (These are the Wormhole token addresses, not necessarily the cannonical version of that token)
     *
     * @param tokenId The Token ID (chain/address)
     * @param chain The chain name or id
     * @returns The Wormhole address on the given chain, null if it does not exist
     */
    async getForeignAsset(tokenId, chain) {
        const context = this.getContext(chain);
        return await context.getForeignAsset(tokenId, chain);
    }
    /**
     * Fetches the address for a token representation on any chain (These are the Wormhole token addresses, not necessarily the cannonical version of that token)
     *
     * @param tokenId The Token ID (chain/address)
     * @param chain The chain name or id
     * @returns The Wormhole address on the given chain
     * @throws Throws if the token does not exist
     */
    async mustGetForeignAsset(tokenId, chain) {
        const context = this.getContext(chain);
        return await context.mustGetForeignAsset(tokenId, chain);
    }
    /**
     * Fetches the number of decimals for a token on a given chain
     *
     * @param tokenId The Token ID (home chain/address)
     * @param chain The chain name or id of the token/representation
     * @returns The number of decimals
     */
    async fetchTokenDecimals(tokenId, chain) {
        const context = this.getContext(chain);
        const repr = await context.mustGetForeignAsset(tokenId, chain);
        return await context.fetchTokenDecimals(repr, chain);
    }
    /**
     * Fetches the native token balance for a wallet
     *
     * @param walletAddress The wallet address
     * @param chain The chain name or id
     * @returns The native balance as a BigNumber
     */
    async getNativeBalance(walletAddress, chain) {
        const context = this.getContext(chain);
        return await context.getNativeBalance(walletAddress, chain);
    }
    /**
     * Fetches the balance of a given token for a wallet
     *
     * @param walletAddress The wallet address
     * @param tokenId The token ID (its home chain and address on the home chain)
     * @param chain The chain name or id
     * @returns The token balance of the wormhole asset as a BigNumber
     */
    async getTokenBalance(walletAddress, tokenId, chain) {
        const context = this.getContext(chain);
        return await context.getTokenBalance(walletAddress, tokenId, chain);
    }
    /**
     * Send a Token Bridge transfer
     *
     * @dev This _must_ be claimed on the destination chain, see {@link WormholeContext#completeTransfer | completeTransfer}
     *
     * @param token The Token Identifier (chain/address) or `'native'` if sending the native token
     * @param amount The token amount to be sent, as a string
     * @param sendingChain The source chain name or id
     * @param senderAddress The address that is dispatching the transfer
     * @param recipientChain The destination chain name or id
     * @param recipientAddress The wallet address where funds will be sent (On solana, this is the associated token account)
     * @param relayerFee A fee that would go to a relayer, if any
     * @param payload Extra bytes that can be passed along with the transfer
     * @returns The transaction receipt
     * @throws If unable to get the signer or contracts, or if there is a problem executing the transaction
     */
    async startTransfer(token, amount, sendingChain, senderAddress, recipientChain, recipientAddress, relayerFee, payload) {
        const context = this.getContext(sendingChain);
        const recipientChainName = this.toChainName(recipientChain);
        if (!payload && recipientChainName === 'sei') {
            let seiContext;
            try {
                seiContext = this.getContext(MAINNET_1.MAINNET_CHAINS.solana);
            }
            catch (_) {
                throw new Error('You attempted to send a transfer to Sei, but the Sei context is not registered. You must import SeiContext from @wormhole-foundation/connect-sdk-sei and pass it in to the Wormhole class constructor');
            }
            const { payload: seiPayload, receiver } = await seiContext.buildSendPayload(token, recipientAddress);
            recipientAddress = receiver || recipientAddress;
            payload = seiPayload || payload;
        }
        if (payload) {
            return context.startTransferWithPayload(token, amount, sendingChain, senderAddress, recipientChain, recipientAddress, payload);
        }
        return context.startTransfer(token, amount, sendingChain, senderAddress, recipientChain, recipientAddress, relayerFee);
    }
    /**
     * Check whether a chain supports automatic relaying
     * @param chain the chain name or chain id
     * @returns boolean representing if automatic relay is available
     */
    supportsSendWithRelay(chain) {
        return !!(this.getContracts(chain)?.relayer &&
            'startTransferWithRelay' in this.getContext(chain));
    }
    /**
     * Sends transaction to the bridge using the relayer
     *
     * @param token The tokenId (chain and address) of the token being sent. Pass in 'native' to send native token
     * @param amount The amount to bridge
     * @param sendingChain The chain name or chain id of the source chain
     * @param senderAddress The address executing the transaction
     * @param recipientChain The chain name or chain id of the destination chain
     * @param recipientAddress The address which will receive funds on the destination chain
     * @param toNativeToken The amount of bridged funds that will be converted to the native gas token on the destination chain
     * @throws If unable to get the signer or contracts, or if there is a problem executing the transaction
     */
    async startTransferWithRelay(token, amount, sendingChain, senderAddress, recipientChain, recipientAddress, toNativeToken, relayerFee) {
        if (!this.supportsSendWithRelay(sendingChain)) {
            throw new Error(`Relayer is not supported on ${this.toChainName(sendingChain)}`);
        }
        const context = this.getContext(sendingChain);
        if (!('startTransferWithRelay' in context)) {
            throw new Error('startTransferWithRelay function not found');
        }
        return context.startTransferWithRelay(token, amount, toNativeToken, sendingChain, senderAddress, recipientChain, recipientAddress);
    }
    /**
     * Redeems funds for a token bridge transfer on the destination chain
     *
     * @param destChain The destination chain name or id
     * @param signedVAA The Signed VAA bytes
     * @param overrides Optional overrides, varies between chains
     * @param payerAddr Optional. The address that pays for the redeem transaction, defaults to the sender address if not specified
     * @returns The transaction receipt
     */
    async completeTransfer(destChain, signedVAA, overrides, receivingAddr) {
        const context = this.getContext(destChain);
        return await context.completeTransfer(destChain, signedVAA, overrides, receivingAddr);
    }
    /**
     * Gets required fields from a ParsedMessage or ParsedRelayerMessage used to fetch a VAA
     *
     * @param txData The ParsedMessage or ParsedRelayerMessage that is the result of a transaction on a source chain
     * @returns The MessageIdentifier collection of fields to uniquely identify a VAA
     */
    getMessageIdentifier(txData) {
        // TODO: wh-connect checks finality first, do we need to?
        const emitterChain = this.toChainId(txData.fromChain);
        const emitterAddress = txData.emitterAddress.startsWith('0x')
            ? txData.emitterAddress.slice(2)
            : txData.emitterAddress;
        return {
            emitterChain: emitterChain,
            emitterAddress,
            sequence: txData.sequence.toString(),
        };
    }
    /**
     * Gets a VAA from the API or Guardian RPC, finality must be met before the VAA will be available. See {@link ChainConfig.finalityThreshold | finalityThreshold} on {@link MAINNET_CONFIG | the config}
     *
     * @param msg The MessageIdentifier used to fetch the VAA
     * @returns The ParsedVAA if available
     */
    async getVAA(msg) {
        const { emitterChain, emitterAddress, sequence } = msg;
        const url = `${this.conf.api}/api/v1/vaas/${emitterChain}/${emitterAddress}/${sequence}`;
        const response = await axios_1.default.get(url);
        if (!response.data.data)
            return;
        const data = response.data.data;
        const vaaBytes = Buffer.from(data.vaa, 'base64');
        return (0, vaa_1.parseVaa)(vaaBytes);
    }
    /**
     * Checks if a transfer has been completed or not
     *
     * @param destChain The destination chain name or id
     * @param signedVAA The Signed VAA bytes
     * @returns True if the transfer has been completed, otherwise false
     */
    async isTransferCompleted(destChain, signedVaa) {
        const context = this.getContext(destChain);
        return await context.isTransferCompleted(destChain, signedVaa);
    }
    /**
     * Format an address to a 32-byte universal address, which can be utilized by the Wormhole contracts
     *
     * @param address The address as a string
     * @returns The address as a 32-byte Wormhole address
     */
    formatAddress(address, chain) {
        const context = this.getContext(chain);
        return context.formatAddress(address);
    }
    /**
     * Parse an address from a 32-byte universal address to a cannonical address
     *
     * @param address The 32-byte wormhole address
     * @returns The address in the blockchain specific format
     */
    parseAddress(address, chain) {
        const context = this.getContext(chain);
        return context.parseAddress(address);
    }
    /**
     * Parses all relevant information from a transaction given the sending tx hash and sending chain
     *
     * @param tx The sending transaction hash
     * @param chain The sending chain name or id
     * @returns The parsed data
     */
    async parseMessageFromTx(tx, chain) {
        const context = this.getContext(chain);
        return await context.parseMessageFromTx(tx, chain);
    }
    /**
     * Get the default config for Mainnet or Testnet
     *
     * @param network 'MAINNET', 'TESTNET' or 'DEVNET'
     * @returns A Wormhole Config
     */
    static getConfig(network) {
        return network === types_1.Network.MAINNET ? MAINNET_1.default : TESTNET_1.default;
    }
}
exports.Wormhole = Wormhole;
//# sourceMappingURL=wormhole.js.map